@@ -1,24 +1,24 @@
-"""This module contains the Model class in Pastas.
-
-"""
+"""This module contains the Model class in Pastas."""
 
 from collections import OrderedDict
+from itertools import combinations
 from logging import getLogger
 from os import getlogin
 
 import numpy as np
-from pandas import date_range, Series, Timedelta, DataFrame, Timestamp
+from pandas import (DataFrame, Series, Timedelta, Timestamp,
+                    date_range, concat, to_timedelta)
 
 from .decorators import get_stressmodel
-from .io.base import dump, _load_model
+from .io.base import _load_model, dump
 from .modelstats import Statistics
 from .noisemodels import NoiseModel
-from .plots import Plotting
+from .modelplots import Plotting
 from .solver import LeastSquares
 from .stressmodels import Constant
 from .timeseries import TimeSeries
-from .utils import _get_dt, _get_time_offset, get_sample, \
-    frequency_is_supported, validate_name
+from .utils import (_get_dt, _get_time_offset, frequency_is_supported,
+                    get_sample, validate_name)
 from .version import __version__
 
 
@@ -41,6 +41,10 @@ class Model:
         Dictionary containing metadata of the oseries, passed on the to
         oseries when creating a pastas TimeSeries object. hence,
         ml.oseries.metadata will give you the metadata.
+    freq: str, optional
+        String with the frequency the stressmodels are simulated. Must
+        be one of the following: (D, h, m, s, ms, us, ns) or a multiple of
+        that e.g. "7D". Default is "D". New in 0.18.0.
 
     Returns
     -------
@@ -53,11 +57,10 @@ class Model:
 
     >>> oseries = pd.Series([1,2,1], index=pd.to_datetime(range(3), unit="D"))
     >>> ml = Model(oseries)
-
     """
 
     def __init__(self, oseries, constant=True, noisemodel=True, name=None,
-                 metadata=None):
+                 metadata=None, freq="D"):
 
         self.logger = getLogger(__name__)
 
@@ -65,10 +68,10 @@ class Model:
         self.oseries = TimeSeries(oseries, settings="oseries",
                                   metadata=metadata)
 
-        if name is None:
+        if name is None and self.oseries.name is not None:
             name = self.oseries.name
-            if name is None:
-                name = 'Observations'
+        elif name is None and self.oseries.name is None:
+            name = 'Observations'
         self.name = validate_name(name)
 
         self.parameters = DataFrame(
@@ -85,8 +88,9 @@ class Model:
         self.settings = {
             "tmin": None,
             "tmax": None,
-            "freq": "D",
-            "warmup": Timedelta(3650, "D"),
+            "freq": freq,
+            "warmup": (3650 * to_timedelta(freq) if freq[0].isdigit()
+                       else Timedelta(3650, freq)),
             "time_offset": Timedelta(0),
             "noise": noisemodel,
             "solver": None,
@@ -109,6 +113,7 @@ class Model:
         self.interpolate_simulation = None
         self.normalize_residuals = False
         self.fit = None
+        self._solve_success = False
 
         # Load other modules
         self.stats = Statistics(self)
@@ -116,8 +121,7 @@ class Model:
         self.plot = self.plots.plot  # because we are lazy
 
     def __repr__(self):
-        """Prints a simple string representation of the model.
-        """
+        """Prints a simple string representation of the model."""
         template = ('{cls}(oseries={os}, name={name}, constant={const}, '
                     'noisemodel={noise})')
         return template.format(cls=self.__class__.__name__,
@@ -159,7 +163,6 @@ class Model:
         See Also
         --------
         pastas.stressmodels
-
         """
         # Method can take multiple stressmodels at once through args
         if isinstance(stressmodel, list):
@@ -172,8 +175,6 @@ class Model:
         else:
             self.stressmodels[stressmodel.name] = stressmodel
             self.parameters = self.get_init_parameters(initial=False)
-            if self.settings["freq"] is None:
-                self._set_freq()
             stressmodel.update_stress(freq=self.settings["freq"])
 
             # Check if stress overlaps with oseries, if not give a warning
@@ -195,7 +196,6 @@ class Model:
         --------
         >>> d = ps.Constant()
         >>> ml.add_constant(d)
-
         """
         self.constant = constant
         self.parameters = self.get_init_parameters(initial=False)
@@ -217,7 +217,6 @@ class Model:
         See Also
         --------
         pastas.transform
-
         """
         transform.set_model(self)
         self.transform = transform
@@ -236,7 +235,6 @@ class Model:
         --------
         >>> n = ps.NoiseModel()
         >>> ml.add_noisemodel(n)
-
         """
         self.noisemodel = noisemodel
         self.noisemodel.set_init_parameters(oseries=self.oseries.series)
@@ -263,15 +261,12 @@ class Model:
         To obtain a list of the stressmodel names type:
 
         >>> ml.get_stressmodel_names()
-
         """
         self.stressmodels.pop(name, None)
         self.parameters = self.get_init_parameters(initial=False)
 
     def del_constant(self):
-        """Method to safely delete the Constant from the Model.
-
-        """
+        """Method to safely delete the Constant from the Model."""
         if self.constant is None:
             self.logger.warning("No constant is present in this model.")
         else:
@@ -279,9 +274,7 @@ class Model:
             self.parameters = self.get_init_parameters(initial=False)
 
     def del_transform(self):
-        """Method to safely delete the transform from the Model.
-
-        """
+        """Method to safely delete the transform from the Model."""
         if self.transform is None:
             self.logger.warning("No transform is present in this model.")
         else:
@@ -289,9 +282,7 @@ class Model:
             self.parameters = self.get_init_parameters(initial=False)
 
     def del_noisemodel(self):
-        """Method to safely delete the noise model from the Model.
-
-        """
+        """Method to safely delete the noise model from the Model."""
         if self.noisemodel is None:
             self.logger.warning("No noisemodel is present in this model.")
         else:
@@ -336,7 +327,6 @@ class Model:
         the initial parameter values are used. This allows the user to
         get an idea of how the simulation looks with only the initial
         parameters and no calibration.
-
         """
         # Default options when tmin, tmax, freq and warmup are not provided.
         if tmin is None and self.settings['tmin']:
@@ -376,7 +366,7 @@ class Model:
             istart += 1
         if self.transform:
             sim = self.transform.simulate(sim, p[istart:istart +
-                                                        self.transform.nparam])
+                                                 self.transform.nparam])
 
         # Respect provided tmin/tmax at this point, since warmup matters for
         # simulation but should not be returned, unless return_warmup=True.
@@ -416,7 +406,6 @@ class Model:
         -------
         res: pandas.Series
             pandas.Series with the residuals series.
-
         """
         # Default options when tmin, tmax, freq and warmup are not provided.
         if tmin is None:
@@ -427,8 +416,7 @@ class Model:
             freq = self.settings["freq"]
 
         # simulate model
-        sim = self.simulate(p, tmin, tmax, freq, warmup,
-                            return_warmup=False)
+        sim = self.simulate(p, tmin, tmax, freq, warmup, return_warmup=False)
 
         # Get the oseries calibration series
         oseries_calib = self.observations(tmin, tmax, freq)
@@ -499,9 +487,8 @@ class Model:
         Warnings
         --------
         This method returns None is no noise model is added to the model.
-
         """
-        if (self.noisemodel is None) or (self.settings["noise"] is False):
+        if self.noisemodel is None or self.settings["noise"] is False:
             self.logger.error("Noise cannot be calculated if there is no "
                               "noisemodel present or is not used during "
                               "parameter estimation.")
@@ -521,7 +508,7 @@ class Model:
 
     def noise_weights(self, p=None, tmin=None, tmax=None, freq=None,
                       warmup=None):
-        """ Internal method to calculate the noise weights."""
+        """Internal method to calculate the noise weights."""
         # Get parameters if none are provided
         if p is None:
             p = self.get_parameters()
@@ -530,8 +517,7 @@ class Model:
         res = self.residuals(p, tmin, tmax, freq, warmup)
 
         # Calculate the weights
-        weights = self.noisemodel.weights(res,
-                                          p[-self.noisemodel.nparam:])
+        weights = self.noisemodel.weights(res, p[-self.noisemodel.nparam:])
 
         return weights
 
@@ -566,7 +552,6 @@ class Model:
         observation. It finds the index closest to sim_index, and then returns
         a selection of the oseries. in the residuals method, the simulation is
         interpolated to the observation-timestamps.
-
         """
         if tmin is None and self.settings['tmin']:
             tmin = self.settings['tmin']
@@ -602,9 +587,9 @@ class Model:
                    noise=None, weights=None, initial=True, fit_constant=True):
         """Method to initialize the model.
 
-        This method is called by the solve-method, but can also be triggered
-        manually. See the solve-method for a description of the arguments.
-
+        This method is called by the solve-method, but can also be
+        triggered manually. See the solve-method for a description of
+        the arguments.
         """
         if noise is None and self.noisemodel:
             noise = True
@@ -713,7 +698,6 @@ class Model:
         --------
         pastas.solver
             Different solver objects are available to estimate parameters.
-
         """
 
         # Initialize the model
@@ -748,96 +732,18 @@ class Model:
 
         self.parameters.optimal = optimal
         self.parameters.stderr = stderr
+        self._solve_success = success  # store for fit_report
 
         if report:
             if isinstance(report, str):
                 output = report
             else:
-                output = "full"
+                output = None
             print(self.fit_report(output=output))
 
-    def set_initial(self, name, value, move_bounds=False):
-        """Method to set the initial value of any parameter.
-
-        Parameters
-        ----------
-        name: str
-            name of the parameter to update.
-        value: float
-            parameters value to use as initial estimate.
-        move_bounds: bool, optional
-            Reset pmin/pmax based on new initial value.
-
-        Examples
-        --------
-        >>> ml.set_initial("constant_d", 10)
-
-        """
-        msg = "Deprecation warning: method is deprecated and will be removed" \
-              " in version 0.17.0. Use ml.set_parameter instead."
-        self.logger.warning(msg)
-
-    def set_vary(self, name, value):
-        """Method to set if the parameter is allowed to vary.
-
-        Parameters
-        ----------
-        name: str
-            name of the parameter to update.
-        value: bool
-            boolean to vary a parameter (True) or not (False).
-
-        Examples
-        --------
-        >>> ml.set_vary("constant_d", False)
-
-        """
-        msg = "Deprecation warning: method is deprecated and will be removed" \
-              " in version 0.17.0. Use ml.set_parameter instead."
-        self.logger.error(msg)
-
-    def set_pmin(self, name, value):
-        """Method to set the minimum value of a parameter.
-
-        Parameters
-        ----------
-        name: str
-            name of the parameter to update.
-        value: float
-            minimum value for the parameter.
-
-        Examples
-        --------
-        >>> ml.set_pmin("constant_d", -10)
-
-        """
-        msg = "Deprecation warning: method is deprecated and will be removed" \
-              " in version 0.17.0. Use ml.set_parameter instead."
-        self.logger.error(msg)
-
-    def set_pmax(self, name, value):
-        """Method to set the maximum values of a parameter.
-
-        Parameters
-        ----------
-        name: str
-            name of the parameter to update.
-        value: float
-            maximum value for the parameter.
-
-        Examples
-        --------
-        >>> ml.set_pmax("constant_d", 10)
-
-        """
-        msg = "Deprecation warning: method is deprecated and will be removed" \
-              " in version 0.17.0. Use ml.set_parameter instead."
-        self.logger.error(msg)
-
     def set_parameter(self, name, initial=None, vary=None, pmin=None,
-                      pmax=None, move_bounds=False):
-        """
-        Method to change the parameter properties.
+                      pmax=None, optimal=None, move_bounds=False):
+        """Method to change the parameter properties.
 
         Parameters
         ----------
@@ -851,6 +757,8 @@ class Model:
             minimum value for the parameter.
         pmax: float, optional
             maximum value for the parameter.
+        optimal: float, optional
+            optimal value for the parameter.
         move_bounds: bool, optional
             Reset pmin/pmax based on new initial value. Of move_bounds=True,
             pmin and pmax must be None.
@@ -865,12 +773,11 @@ class Model:
         It is highly recommended to use this method to set parameter
         properties. Changing the parameter properties directly in the
         parameter `DataFrame` may not work as expected.
-
         """
         if name not in self.parameters.index:
-            msg = "parameter {} is not present in the model".format(name)
-            self.logger.error(msg)
-            raise KeyError(msg)
+            msg = "parameter %s is not present in the model"
+            self.logger.error(msg, name)
+            raise KeyError(msg, name)
 
         # Because either of the following is not necessarily present
         noisemodel = self.noisemodel.name if self.noisemodel else "NotPresent"
@@ -911,41 +818,8 @@ class Model:
         if pmax is not None:
             obj._set_pmax(name, pmax)
             self.parameters.loc[name, "pmax"] = pmax
-
-    def _set_freq(self):
-        """Internal method to set the frequency in the settings. This is
-        method is not yet applied and is for future development.
-
-        """
-        freqs = set()
-        if self.oseries.freq:
-            # when the oseries has a constant frequency, us this
-            freqs.add(self.oseries.freq)
-        else:
-            # otherwise determine frequency from the stressmodels
-            for stressmodel in self.stressmodels.values():
-                if stressmodel.stress:
-                    for stress in stressmodel.stress:
-                        if stress.settings['freq']:
-                            # first check the frequency, and use this
-                            freqs.add(stress.settings['freq'])
-                        elif stress.freq_original:
-                            # if this is not available, and the original
-                            # frequency is, take the original frequency
-                            freqs.add(stress.freq_original)
-
-        if len(freqs) == 1:
-            # if there is only one frequency, use this frequency
-            self.settings["freq"] = next(iter(freqs))
-        elif len(freqs) > 1:
-            # if there are more frequencies, take the highest (lowest dt)
-            freqs = list(freqs)
-            dt = np.array([_get_dt(f) for f in freqs])
-            self.settings["freq"] = freqs[np.argmin(dt)]
-        else:
-            self.logger.info("Frequency of model cannot be determined. "
-                             "Frequency is set to daily")
-            self.settings["freq"] = "D"
+        if optimal is not None:
+            self.parameters.loc[name, "optimal"] = optimal
 
     def _get_time_offset(self, freq):
         """Internal method to get the time offsets from the stressmodels.
@@ -960,9 +834,6 @@ class Model:
 
         Method to check if the StressModel timestamps match
         (e.g. similar hours)
-
-
-
         """
         time_offsets = set()
         for stressmodel in self.stressmodels.values():
@@ -975,8 +846,8 @@ class Model:
                     if np.any(mask):
                         time_offsets.add(_get_time_offset(t[mask][0], freq))
         if len(time_offsets) > 1:
-            msg = ("The time-offset with the frequency is not the same "
-                   "for all stresses.")
+            msg = "The time-offset with the frequency is not the same for " \
+                  "all stresses."
             self.logger.error(msg)
             raise (Exception(msg))
         if len(time_offsets) == 1:
@@ -1009,7 +880,6 @@ class Model:
         sim_index: pandas.DatetimeIndex
             Pandas DatetimeIndex instance with the datetimes values for
             which the model is simulated.
-
         """
         # Check if any of the settings are updated
         for key, setting in zip([tmin, tmax, freq, warmup],
@@ -1059,7 +929,6 @@ class Model:
 
             1. A pandas timestamp is made from the string
             2. if use_oseries is True, tmin is checked against oseries.
-
         """
         # Get tmin from the oseries
         if use_oseries:
@@ -1121,7 +990,6 @@ class Model:
 
         A detailed description of dealing with tmax and timesteps
         in general can be found in the developers section of the docs.
-
         """
         # Get tmax from the oseries
         if use_oseries:
@@ -1161,29 +1029,28 @@ class Model:
         -------
         parameters: pandas.DataFrame
             pandas.Dataframe with the parameters.
-
         """
         if noise is None:
             noise = self.settings['noise']
 
-        parameters = DataFrame(columns=self.parameters.columns)
+        frames = [DataFrame(columns=self.parameters.columns)]
+
         for sm in self.stressmodels.values():
-            parameters = parameters.append(sm.parameters, sort=False)
+            frames.append(sm.parameters)
         if self.constant:
-            parameters = parameters.append(self.constant.parameters,
-                                           sort=False)
+            frames.append(self.constant.parameters)
         if self.transform:
-            parameters = parameters.append(self.transform.parameters,
-                                           sort=False)
+            frames.append(self.transform.parameters)
         if self.noisemodel and noise:
-            parameters = parameters.append(self.noisemodel.parameters,
-                                           sort=False)
+            frames.append(self.noisemodel.parameters)
+
+        parameters = concat(frames)
+        parameters = parameters.infer_objects()
 
         # Set initial parameters to optimal parameters from model
         if not initial:
-            paramold = self.parameters.optimal
-            parameters.initial.update(paramold)
-            parameters.optimal.update(paramold)
+            parameters.initial.update(self.parameters.optimal)
+            parameters.optimal.update(self.parameters.optimal)
 
         return parameters
 
@@ -1202,7 +1069,6 @@ class Model:
         -------
         p: numpy.ndarray
             Numpy array with the parameters used in the time series model.
-
         """
         if name:
             p = self.parameters.loc[self.parameters.name == name]
@@ -1210,18 +1076,40 @@ class Model:
             p = self.parameters
 
         if p.optimal.hasnans:
-            self.logger.warning(
-                "Model is not optimized yet, initial parameters are used.")
+            self.logger.warning("Model is not optimized yet, initial "
+                                "parameters are used.")
             parameters = p.initial
         else:
             parameters = p.optimal
 
-        return parameters.values
+        return parameters.to_numpy(dtype=float)
 
     def get_stressmodel_names(self):
-        """Returns list of stressmodel names"""
+        """Returns list of stressmodel names."""
         return list(self.stressmodels.keys())
 
+    @get_stressmodel
+    def get_stressmodel_settings(self, name):
+        """Method to obtain the time series settings for a stress model.
+
+        Parameters
+        ----------
+        name: str, optional
+            string with the name of the pastas.stressmodel object.
+
+        Returns
+        -------
+        dict or None
+            Dictionary with the settings or "None" of no stress are present,
+            e.g., for a step model that uses no stress.
+        """
+        sm = self.stressmodels[name]
+        if len(sm.stress) == 0:
+            settings = None
+        else:
+            settings = {stress.name: stress.settings for stress in sm.stress}
+        return settings
+
     @get_stressmodel
     def get_contribution(self, name, tmin=None, tmax=None, freq=None,
                          warmup=None, istress=None, return_warmup=False,
@@ -1258,7 +1146,6 @@ class Model:
         -------
         contrib: pandas.Series
             Pandas Series with the contribution.
-
         """
         if p is None:
             p = self.get_parameters(name)
@@ -1277,7 +1164,7 @@ class Model:
         # use warmup
         if tmin:
             tmin_warm = (Timestamp(tmin) - warmup).floor(freq) + \
-                        self.settings["time_offset"]
+                self.settings["time_offset"]
         else:
             tmin_warm = None
 
@@ -1314,7 +1201,6 @@ class Model:
         pastas.model.Model.get_contribution
             This method is called to get the individual contributions,
             kwargs are passed on to this method.
-
         """
         contribs = []
         for name in self.stressmodels:
@@ -1345,7 +1231,6 @@ class Model:
         -------
         contrib: pandas.Series
             Pandas Series with the contribution.
-
         """
         sim = self.simulate(tmin=tmin, tmax=tmax)
         # calculate what the simulation without the transform is
@@ -1377,10 +1262,9 @@ class Model:
         Returns
         -------
         response: pandas.Series
-
         """
         if self.stressmodels[name].rfunc is None:
-            self.logger.warning("Stressmodel {} has no rfunc".format(name))
+            self.logger.warning("Stressmodel %s has no rfunc.", name)
             return None
         else:
             block_or_step = getattr(self.stressmodels[name].rfunc,
@@ -1394,12 +1278,17 @@ class Model:
         response = block_or_step(p, dt, **kwargs)
 
         if add_0:
+            if isinstance(dt, np.ndarray):
+                t = dt
+            else:
+                t = np.linspace(0, response.size * dt, response.size + 1)
             response = np.insert(response, 0, 0.0)
-
-        if isinstance(dt, np.ndarray):
-            t = dt
         else:
-            t = np.linspace(dt, response.size * dt, response.size)
+            if isinstance(dt, np.ndarray):
+                t = dt
+            else:
+                t = np.linspace(dt, response.size * dt, response.size)
+
         response = Series(response, index=t, name=name)
         response.index.name = "Time [days]"
 
@@ -1429,7 +1318,6 @@ class Model:
         b: pandas.Series
             Pandas.Series with the block response. The index is based on the
             frequency that is present in the model.settings.
-
         """
         return self._get_response(block_or_step="block", name=name, dt=dt,
                                   p=p, add_0=add_0, **kwargs)
@@ -1458,7 +1346,6 @@ class Model:
         s: pandas.Series
             Pandas.Series with the step response. The index is based on the
             frequency that is present in the model.settings.
-
         """
         return self._get_response(block_or_step="step", name=name, dt=dt,
                                   p=p, add_0=add_0, **kwargs)
@@ -1490,10 +1377,9 @@ class Model:
 
         This means that after 1053 days, 99% of the response of the
         groundwater levels to a recharge pulse has taken place.
-
         """
         if self.stressmodels[name].rfunc is None:
-            self.logger.warning("Stressmodel {} has no rfunc".format(name))
+            self.logger.warning("Stressmodel %s has no rfunc", name)
             return None
         else:
             if p is None:
@@ -1537,7 +1423,6 @@ class Model:
         stress: pandas.Series or list of pandas.Series
             If one stress is present, a pandas Series is returned. If more
             are present, a list of pandas Series is returned.
-
         """
         if p is None:
             p = self.get_parameters(name)
@@ -1556,7 +1441,7 @@ class Model:
         # use warmup
         if tmin:
             tmin_warm = (Timestamp(tmin) - warmup).floor(freq) + \
-                        self.settings["time_offset"]
+                self.settings["time_offset"]
         else:
             tmin_warm = None
 
@@ -1577,7 +1462,6 @@ class Model:
         -------
         file_info: dict
             dictionary with file information.
-
         """
         # Check if file_info already exists
         if hasattr(self, "file_info"):
@@ -1595,7 +1479,7 @@ class Model:
 
         return file_info
 
-    def fit_report(self, output="full"):
+    def fit_report(self, output="basic", warnings=True, warnbounds=None):
         """Method that reports on the fit after a model is optimized.
 
         Parameters
@@ -1603,6 +1487,12 @@ class Model:
         output: str, optional
             If any other value than "full" is provided, the parameter
             correlations will be removed from the output.
+        warnings : bool, optional
+            print warnings in case of optimization failure, parameters
+            hitting bounds, or length of responses exceeding calibration
+            period.
+        warnbounds : bool, optional
+            deprecated, use warnings instead
 
         Returns
         -------
@@ -1621,7 +1511,6 @@ class Model:
         The reported values for the fit use the residuals time series where
         possible. If interpolation is used this means that the result may
         slightly differ compared to using ml.simulate() and ml.observations().
-
         """
         model = {
             "nfev": self.fit.nfev,
@@ -1635,98 +1524,188 @@ class Model:
         }
 
         fit = {
-            "EVP": "{:.2f}".format(self.stats.evp()),
-            "R2": "{:.2f}".format(self.stats.rsq()),
-            "RMSE": "{:.2f}".format(self.stats.rmse()),
-            "AIC": "{:.2f}".format(self.stats.aic() if
-                                   self.settings["noise"] else np.nan),
-            "BIC": "{:.2f}".format(self.stats.bic() if
-                                   self.settings["noise"] else np.nan),
-            "Obj": "{:.2f}".format(self.fit.obj_func),
-            "___": "", "Interpolated": "Yes" if self.interpolate_simulation
-            else "No",
+            "EVP": f"{self.stats.evp():.2f}",
+            "R2": f"{self.stats.rsq():.2f}",
+            "RMSE": f"{self.stats.rmse():.2f}",
+            "AIC": f"{self.stats.aic():.2f}",
+            "BIC": f"{self.stats.bic():.2f}",
+            "Obj": f"{self.fit.obj_func:.2f}",
+            "___": "",
+            "Interp.": "Yes" if self.interpolate_simulation else "No",
         }
 
+        if warnbounds:
+            DeprecationWarning("The 'warnbounds' argument is deprecated. "
+                               "Use warnings=True instead.")
+
         parameters = self.parameters.loc[:, ["optimal", "stderr",
                                              "initial", "vary"]]
         stderr = parameters.loc[:, "stderr"] / parameters.loc[:, "optimal"]
-        parameters.loc[:, "stderr"] = stderr.abs().apply("\u00B1{:.2%}".format)
+        parameters.loc[:, "stderr"] = stderr.abs().apply("±{:.2%}".format)
 
         # Determine the width of the fit_report based on the parameters
-        width = len(parameters.__str__().split("\n")[1])
+        width = len(parameters.to_string().split("\n")[1])
         string = "{:{fill}{align}{width}}"
 
         # Create the first header with model information and stats
-        w = max(width - 44, 0)
-        header = "Fit report {name:<16}{string}Fit Statistics\n" \
-                 "{line}\n".format(
-            name=self.name[:14],
-            string=string.format("", fill=' ', align='>', width=w),
-            line=string.format("", fill='=', align='>', width=width))
+        wspace = max(width - (11 + 14 + len(self.name)), 1)
+        mspace = width - wspace - (11 + 14)
+        header = (
+            f"Fit report {self.name:<{mspace}.{mspace}}"
+            f"{string.format('', fill=' ', align='>', width=wspace)}"
+            f"Fit Statistics\n"
+            f"{string.format('', fill='=', align='>', width=width)}\n"
+        )
 
         basic = ""
-        w = max(width - 45, 0)
+        len_val4 = max([len(v) for v in fit.values()])
+        wspace = width - (8 + 23 + 9 + len_val4)
         for (val1, val2), (val3, val4) in zip(model.items(), fit.items()):
-            val4 = string.format(val4, fill=' ', align='>', width=w)
-            basic += "{:<8} {:<22} {:<12} {:}\n".format(val1, val2, val3, val4)
+            basic += (
+                f"{val1:<8}{val2:<23}{val3:<9}"
+                f"{val4:>{wspace+len_val4}}\n"
+            )
 
         # Create the parameters block
-        parameters = "\nParameters ({n_param} were optimized)\n{line}\n" \
-                     "{parameters}".format(
-            n_param=parameters.vary.sum(),
-            line=string.format("", fill='=', align='>', width=width),
-            parameters=parameters)
+        params = f"\nParameters ({parameters.vary.sum()} optimized)\n" \
+                 f"{string.format('', fill='=', align='>', width=width)}\n" \
+                 f"{parameters.to_string()}"
 
         if output == "full":
-            cor = {}
-            pcor = self.fit.pcor
-            for idx in pcor:
-                for col in pcor:
-                    if (np.abs(pcor.loc[idx, col]) > 0.5) and (idx != col) \
-                            and ((col, idx) not in cor.keys()):
-                        cor[(idx, col)] = pcor.loc[idx, col].round(2)
+            cor = DataFrame(columns=["value"])
+            for idx, col in combinations(self.fit.pcor, 2):
+                if np.abs(self.fit.pcor.loc[idx, col]) > 0.5:
+                    cor.loc[f"{idx} {col}"] = self.fit.pcor.loc[idx, col]
 
-            cor = DataFrame(data=cor.values(), index=cor.keys(),
-                            columns=["rho"])
-            correlations = "\n\nParameter correlations |rho| > 0.5\n{}" \
-                           "\n{}".format(string.format("", fill='=', align='>',
-                                                       width=width),
-                                         cor.to_string(header=False))
+            corr = f"\n\nParameter correlations |rho| > 0.5\n" \
+                   f"{string.format('', fill='=', align='>', width=width)}" \
+                   f"\n{cor.to_string(float_format='%.2f', header=False)}"
         else:
-            correlations = ""
+            corr = ""
 
-        report = "{header}{basic}{parameters}{correlations}".format(
-            header=header, basic=basic, parameters=parameters,
-            correlations=correlations)
+        if warnings:
+            msg = []
+            # model optmization unsuccesful
+            if not self._solve_success:
+                msg.append("Model parameters could not be estimated well")
+
+            # parameter bound warnings
+            lowerhit, upperhit = self._check_parameters_bounds()
+            nhits = upperhit.sum() + lowerhit.sum()
+
+            if nhits > 0:
+                for p in upperhit.index:
+                    if upperhit.loc[p]:
+                        msg.append(
+                            f"Parameter '{p}' on upper bound: "
+                            f"{self.parameters.loc[p, 'pmax']:.2e}"
+                        )
+                    elif lowerhit.loc[p]:
+                        msg.append(
+                            f"Parameter '{p}' on lower bound: "
+                            f"{self.parameters.loc[p, 'pmin']:.2e}"
+                        )
+            # check response t_cutoff vs length calibration period
+            response_tmax_check = self._check_response_tmax()
+            if (~response_tmax_check["check_ok"]).any():
+                mask = ~response_tmax_check["check_ok"]
+                for i in response_tmax_check.loc[mask].index:
+                    msg.append(f"Response tmax for '{i}' > "
+                               "than calibration period")
+
+            # create message
+            if len(msg) > 0:
+                msg = [
+                    f"\n\nWarnings! ({len(msg)})\n"
+                    f"{string.format('', fill='=', align='>', width=width)}"
+                ] + msg
+                warnings = "\n".join(msg)
+            else:
+                warnings = ""
+        else:
+            warnings = ""
+
+        report = f"{header}{basic}{params}{warnings}{corr}"
 
         return report
 
-    def _check_parameters_bounds(self, alpha=0.01):
-        """Internal method to check if the optimal parameters are close to
-        pmin or pmax.
+    def _check_response_tmax(self, cutoff=None):
+        """Internal method to check whether response tmax is smaller than
+        calibration period.
 
         Parameters
         ----------
-        alpha: float, optional
-            value between 0 and 1 to determine if the parameters is close to
-            the maximum or minimum is determined as the percentage of the
-            parameter range.
+        cutoff : float, optional
+            cutoff for response function, by default None, which uses
+            cutoff defined for each response function
 
         Returns
         -------
-        pmin: pandas.Series
-            pandas series with boolean values of the parameters that are
-            close to the minimum values.
-        pmax: pandas.Series
-            pandas series with boolean values of the parameters that are
-            close to the maximum values.
-
+        check : pandas.DataFrame
+            dataframe containing length calibration period, response tmax
+            for each stressmodel, and check result
         """
-        prange = self.parameters.pmax - self.parameters.pmin
-        pnorm = (self.parameters.optimal - self.parameters.pmin) / prange
-        pmax = pnorm > 1 - alpha
-        pmin = pnorm < alpha
-        return pmin, pmax
+
+        len_oseries_calib = (self.settings["tmax"] -
+                             self.settings["tmin"]).days
+
+        check = DataFrame(index=self.stressmodels.keys(),
+                          columns=["len_oseries_calib",
+                                   "response_tmax",
+                                   "check_ok"])
+        check["len_oseries_calib"] = len_oseries_calib
+
+        for sm_name in self.stressmodels:
+            check.loc[sm_name, "response_tmax"] = self.get_response_tmax(
+                sm_name, cutoff=cutoff)
+
+        check["check_ok"] = check["response_tmax"] < check["len_oseries_calib"]
+
+        return check
+
+    def _check_parameters_bounds(self):
+        """Internal method to check if the optimal parameters are close to pmin
+        or pmax.
+
+        Returns
+        -------
+        lowerhit: pandas.Series
+            pandas series with boolean values of the parameters that are
+            close to the minimum (pmin) values.
+        upperhit: pandas.Series
+            pandas series with boolean values of the parameters that are
+            close to the maximum (pmax) values.
+        """
+        upperhit = Series(index=self.parameters.index, dtype=bool)
+        lowerhit = Series(index=self.parameters.index, dtype=bool)
+
+        for p in self.parameters.index:
+            pmax = self.parameters.loc[p, "pmax"]
+            pmin = self.parameters.loc[p, "pmin"]
+
+            # calculate atol based on minimum, with max 1e-8
+            # otherwise set 1 order of magnitude lower than minimum value
+            if pmin == 0.0 or np.isnan(pmin):
+                atol = 1e-8
+            else:
+                atol = np.min(
+                    [1e-8, 10 ** (np.floor(np.log10(np.abs(pmin))) - 1)])
+
+            # deal with NaNs in parameter bounds
+            if np.isnan(pmax):
+                pmax = np.inf
+            if np.isnan(pmin):
+                pmax = -np.inf
+
+            # determine hits
+            upperhit.loc[p] = np.allclose(
+                self.parameters.loc[p, "optimal"], pmax,
+                atol=atol, rtol=1e-5)
+            lowerhit.loc[p] = np.allclose(
+                self.parameters.loc[p, "optimal"], pmin,
+                atol=atol, rtol=1e-5)
+
+        return lowerhit, upperhit
 
     def to_dict(self, series=True, file_info=True):
         """Method to export a model to a dictionary.
@@ -1743,7 +1722,6 @@ class Model:
         Helper function for the self.to_file method. To increase backward
         compatibility most attributes are stored in dictionaries that can be
         updated when a model is created.
-
         """
 
         # Create a dictionary to store all data
@@ -1798,7 +1776,6 @@ class Model:
         See Also
         --------
         :mod:`pastas.io.dump`
-
         """
 
         # Get dicts for all data sources
@@ -1824,7 +1801,6 @@ class Model:
         Examples
         --------
         >>> ml_copy = ml.copy(name="new_name")
-
         """
         if name is None:
             name = self.name + "_copy"
