@@ -10,16 +10,12 @@ These methods may be used as follows:
 or
 
 >>> ml.stats.rmse()
-
 """
 
 from logging import getLogger
 
-from numpy import sqrt, log, ones, nan
-from pandas import Timedelta
-
-from pastas.decorators import PastasDeprecationWarning
-from pastas.stats.core import mean, var, std
+from numpy import abs, average, log, nan, sqrt
+from pastas.stats.core import _get_weights, mean, std, var
 
 __all__ = ["rmse", "sse", "mae", "nse", "evp", "rsq", "bic", "aic",
            "pearsonr", "kge_2012"]
@@ -29,7 +25,7 @@ logger = getLogger(__name__)
 # Absolute Error Metrics
 
 def mae(obs=None, sim=None, res=None, missing="drop", weighted=False,
-        max_gap=90):
+        max_gap=30):
     """Compute the (weighted) Mean Absolute Error (MAE).
 
     Parameters
@@ -50,7 +46,7 @@ def mae(obs=None, sim=None, res=None, missing="drop", weighted=False,
     max_gap: int, optional
         maximum allowed gap period in days to use for the computation of the
         weights. All time steps larger than max_gap are replace with the
-        mean weight. Default value is 90 days.
+        max_gap value. Default value is 30 days.
 
     Notes
     -----
@@ -60,7 +56,6 @@ def mae(obs=None, sim=None, res=None, missing="drop", weighted=False,
     .. math:: \\text{MAE} = \\sum_{i=1}^{N} w_i |x_i - y_i|
 
     where :math:`N` is the number of observations in the observed time series.
-
     """
     if res is None:
         res = sim - obs
@@ -69,22 +64,16 @@ def mae(obs=None, sim=None, res=None, missing="drop", weighted=False,
         res = res.dropna()
 
     # Return nan if the time indices of the sim and obs don't match
-    if res.index.size is 0:
+    if res.index.size == 0:
         logger.warning("Time indices of the sim and obs don't match.")
         return nan
 
-    if weighted:
-        w = (res.index[1:] - res.index[:-1]).to_numpy() / Timedelta("1D")
-        w[w > max_gap] = w[w <= max_gap].mean()
-    else:
-        w = ones(res.index.size - 1)
-    w /= w.sum()
-
-    return (w * res[1:].abs()).sum()
+    w = _get_weights(res, weighted=weighted, max_gap=max_gap)
+    return (w * abs(res.to_numpy())).sum()
 
 
 def rmse(obs=None, sim=None, res=None, missing="drop", weighted=False,
-         max_gap=90):
+         max_gap=30):
     """Compute the (weighted) Root Mean Squared Error (RMSE).
 
     Parameters
@@ -101,20 +90,19 @@ def rmse(obs=None, sim=None, res=None, missing="drop", weighted=False,
         supported now.
     weighted: bool, optional
         Weight the values by the normalized time step to account for
-        irregular time series. Default is True.
+        irregular time series. Default is False.
     max_gap: int, optional
         maximum allowed gap period in days to use for the computation of the
         weights. All time steps larger than max_gap are replace with the
-        mean weight. Default value is 90 days.
+        max_gap value. Default value is 30 days.
 
     Notes
     -----
     Computes the Root Mean Squared Error (RMSE) as follows:
 
-    .. math:: \\text{RMSE} = \\sqrt{\\sum_{i=1}^{N} w_i(n_i- \\bar{n})^2}
+    .. math:: \\text{RMSE} = \\sqrt{\\sum_{i=1}^{N} w_i n_i^2}
 
     where :math:`N` is the number of residuals :math:`n`.
-
     """
     if res is None:
         res = sim - obs
@@ -123,18 +111,12 @@ def rmse(obs=None, sim=None, res=None, missing="drop", weighted=False,
         res = res.dropna()
 
     # Return nan if the time indices of the sim and obs don't match
-    if res.index.size is 0:
+    if res.index.size == 0:
         logger.warning("Time indices of the sim and obs don't match.")
         return nan
 
-    if weighted:
-        w = (res.index[1:] - res.index[:-1]).to_numpy() / Timedelta("1D")
-        w[w > max_gap] = w[w <= max_gap].mean()
-    else:
-        w = ones(res.index.size - 1)
-    w /= w.sum()
-
-    return sqrt((res[1:] ** 2 * w).sum())
+    w = _get_weights(res, weighted=weighted, max_gap=max_gap)
+    return sqrt((w * res.to_numpy() ** 2).sum())
 
 
 def sse(obs=None, sim=None, res=None, missing="drop"):
@@ -160,7 +142,6 @@ def sse(obs=None, sim=None, res=None, missing="drop"):
     .. math:: \\text{SSE} = \\sum(r^2)
 
     Where :math:`r` are the residuals.
-
     """
     if res is None:
         res = (sim - obs)
@@ -169,50 +150,16 @@ def sse(obs=None, sim=None, res=None, missing="drop"):
         res = res.dropna()
 
     # Return nan if the time indices of the sim and obs don't match
-    if res.index.size is 0:
+    if res.index.size == 0:
         logger.warning("Time indices of the sim and obs don't match.")
         return nan
 
-    return (res ** 2).sum()
-
-
-@PastasDeprecationWarning
-def avg_dev(sim, obs, missing="drop"):
-    """Average deviation of the residuals.
-
-    Parameters
-    ----------
-    sim: pandas.Series
-        Series with the simulated values.
-    obs: pandas.Series
-        Series with the observed values.
-    missing: str, optional
-        string with the rule to deal with missing values. Only "drop" is
-        supported now.
-
-    Notes
-    -----
-    .. math:: avg_{dev} = \\frac{\\sum(r)}{n}
-
-    where :math:`n` is the number of residuals :math:`r`.
-
-    """
-    res = (sim - obs)
-
-    if missing == "drop":
-        res = res.dropna()
-
-    # Return nan if the time indices of the sim and obs don't match
-    if res.index.size is 0:
-        logger.warning("Time indices of the sim and obs don't match.")
-        return nan
-
-    return res.mean()
+    return (res.to_numpy() ** 2).sum()
 
 
 # Percentage Error Metrics
 
-def pearsonr(obs, sim, missing="drop", weighted=False, max_gap=90):
+def pearsonr(obs, sim, missing="drop", weighted=False, max_gap=30):
     """Compute the (weighted) Pearson correlation (r).
 
     Parameters
@@ -226,11 +173,11 @@ def pearsonr(obs, sim, missing="drop", weighted=False, max_gap=90):
         observed series. Only "drop" is supported now.
     weighted: bool, optional
         Weight the values by the normalized time step to account for
-        irregular time series. Default is True.
+        irregular time series. Default is False.
     max_gap: int, optional
         maximum allowed gap period in days to use for the computation of the
         weights. All time steps larger than max_gap are replace with the
-        mean weight. Default value is 90 days.
+        max_gap value. Default value is 30 days.
 
     Notes
     -----
@@ -243,28 +190,20 @@ def pearsonr(obs, sim, missing="drop", weighted=False, max_gap=90):
     Where :math:`x` is is observed time series, :math:`y` the simulated
     time series, and :math:`N` the number of observations in the observed
     time series.
-
     """
     if missing == "drop":
         obs = obs.dropna()
 
-    if weighted:
-        w = (obs.index[1:] - obs.index[:-1]).to_numpy() / Timedelta("1D")
-        w[w > max_gap] = w[w <= max_gap].mean()
-    else:
-        w = ones(obs.index.size - 1)
-
-    w /= w.sum()
-
-    sim = sim.reindex(obs.index).dropna()
+    w = _get_weights(obs, weighted=weighted, max_gap=max_gap)
+    sim = sim.reindex(obs.index).dropna().to_numpy()
 
     # Return nan if the time indices of the sim and obs don't match
-    if sim.index.size is 0:
+    if sim.size == 0:
         logger.warning("Time indices of the sim and obs don't match.")
         return nan
 
-    sim = sim[1:] - mean(sim, weighted=weighted, max_gap=max_gap)
-    obs = obs[1:] - mean(obs, weighted=weighted, max_gap=max_gap)
+    sim = sim - average(sim, weights=w)
+    obs = obs.to_numpy() - average(obs.to_numpy(), weights=w)
 
     r = (w * sim * obs).sum() / \
         sqrt((w * sim ** 2).sum() * (w * obs ** 2).sum())
@@ -272,7 +211,7 @@ def pearsonr(obs, sim, missing="drop", weighted=False, max_gap=90):
     return r
 
 
-def evp(obs, sim=None, res=None, missing="drop", weighted=False, max_gap=90):
+def evp(obs, sim=None, res=None, missing="drop", weighted=False, max_gap=30):
     """Compute the (weighted) Explained Variance Percentage (EVP).
 
     Parameters
@@ -289,11 +228,11 @@ def evp(obs, sim=None, res=None, missing="drop", weighted=False, max_gap=90):
         supported now.
     weighted: bool, optional
         If weighted is True, the variances are computed using the time
-        step between observations as weights. Default is True.
+        step between observations as weights. Default is False.
     max_gap: int, optional
         maximum allowed gap period in days to use for the computation of the
         weights. All time steps larger than max_gap are replace with the
-        mean weight. Default value is 90 days.
+        max_gap value. Default value is 30 days.
 
     Notes
     -----
@@ -313,8 +252,7 @@ def evp(obs, sim=None, res=None, missing="drop", weighted=False, max_gap=90):
        M. Bakker, T.N. Olsthoorn, D.G. Cirkel, I. Leunk, F. Schaars, and D.C.
        von Asmuth. 2012. Software for hydrogeologic time series analysis,
        interfacing data with physical insight. Environmental Modelling &
-       Software 38: 178–190.
-
+       Software 38: 178–130.
     """
     if res is None:
         res = sim - obs
@@ -323,7 +261,7 @@ def evp(obs, sim=None, res=None, missing="drop", weighted=False, max_gap=90):
         res = res.dropna()
 
     # Return nan if the time indices of the sim and obs don't match
-    if res.index.size is 0:
+    if res.index.size == 0:
         logger.warning("Time indices of the sim and obs don't match.")
         return nan
 
@@ -334,7 +272,7 @@ def evp(obs, sim=None, res=None, missing="drop", weighted=False, max_gap=90):
                          var(obs, weighted=weighted, max_gap=max_gap))) * 100
 
 
-def nse(obs, sim=None, res=None, missing="drop", weighted=False, max_gap=90):
+def nse(obs, sim=None, res=None, missing="drop", weighted=False, max_gap=30):
     """Compute the (weighted) Nash-Sutcliffe Efficiency (NSE).
 
     Parameters
@@ -355,18 +293,19 @@ def nse(obs, sim=None, res=None, missing="drop", weighted=False, max_gap=90):
     max_gap: int, optional
         maximum allowed gap period in days to use for the computation of the
         weights. All time steps larger than max_gap are replace with the
-        mean weight. Default value is 90 days.
+        max_gap value. Default value is 30 days.
 
     Notes
     -----
+    NSE computed according to [nash_1970]_
+
     .. math:: \\text{NSE} = 1 - \\frac{\\sum(h_s-h_o)^2}{\\sum(h_o-\\mu_{h,o})}
 
     References
     ----------
     .. [nash_1970] Nash, J. E., & Sutcliffe, J. V. (1970). River flow
        forecasting through conceptual models part I-A discussion of
-       principles. Journal of hydrology, 10(3), 282-290.
-
+       principles. Journal of hydrology, 10(3), 282-230.
     """
     if res is None:
         res = sim - obs
@@ -375,23 +314,19 @@ def nse(obs, sim=None, res=None, missing="drop", weighted=False, max_gap=90):
         res = res.dropna()
 
     # Return nan if the time indices of the sim and obs don't match
-    if res.index.size is 0:
+    if res.index.size == 0:
         logger.warning("Time indices of the sim and obs don't match.")
         return nan
 
-    if weighted:
-        w = (obs.index[1:] - obs.index[:-1]).to_numpy() / Timedelta("1D")
-        w[w > max_gap] = w[w <= max_gap].mean()
-    else:
-        w = ones(obs.index.size - 1)
+    w = _get_weights(res, weighted=weighted, max_gap=max_gap)
+    mu = average(obs.to_numpy(), weights=w)
 
-    w /= w.sum()
-    mu = mean(obs, weighted=weighted, max_gap=max_gap)
-
-    return 1 - (w * res[1:] ** 2).sum() / (w * (obs[1:] - mu) ** 2).sum()
+    return 1 - (w * res.to_numpy() ** 2).sum() / \
+           (w * (obs.to_numpy() - mu) ** 2).sum()
 
 
-def rsq(obs, sim=None, res=None, missing="drop", nparam=None):
+def rsq(obs, sim=None, res=None, missing="drop", weighted=False, max_gap=30,
+        nparam=None):
     """Compute R-squared, possibly adjusted for the number of free parameters.
 
     Parameters
@@ -403,11 +338,18 @@ def rsq(obs, sim=None, res=None, missing="drop", nparam=None):
     res: pandas.Series
         Series with the residual values. If time series for the residuals
         are provided, the sim and obs arguments are ignored.
-    nparam: int, optional
-        number of calibrated parameters.
     missing: str, optional
         string with the rule to deal with missing values. Only "drop" is
         supported now.
+    weighted: bool, optional
+        If weighted is True, the variances are computed using the time
+        step between observations as weights. Default is False.
+    max_gap: int, optional
+        maximum allowed gap period in days to use for the computation of the
+        weights. All time steps larger than max_gap are replace with the
+        max_gap value. Default value is 30 days.
+    nparam: int, optional
+        number of calibrated parameters.
 
     Notes
     -----
@@ -428,12 +370,14 @@ def rsq(obs, sim=None, res=None, missing="drop", nparam=None):
         res = res.dropna()
 
     # Return nan if the time indices of the sim and obs don't match
-    if res.index.size is 0:
+    if res.index.size == 0:
         logger.warning("Time indices of the sim and obs don't match.")
         return nan
 
-    rss = (res ** 2.0).sum()
-    tss = ((obs - obs.mean()) ** 2.0).sum()
+    w = _get_weights(res, weighted=weighted, max_gap=max_gap)
+    mu = average(obs.to_numpy(), weights=w)
+    rss = (w * res.to_numpy() ** 2.0).sum()
+    tss = (w * (obs.to_numpy() - mu) ** 2.0).sum()
 
     if nparam:
         return 1.0 - (obs.size - 1.0) / (obs.size - nparam) * rss / tss
@@ -482,11 +426,13 @@ def bic(obs=None, sim=None, res=None, missing="drop", nparam=1):
         res = res.dropna()
 
     # Return nan if the time indices of the sim and obs don't match
-    if res.index.size is 0:
+    if res.index.size == 0:
         logger.warning("Time indices of the sim and obs don't match.")
         return nan
 
-    return -2.0 * log((res ** 2.0).sum()) + nparam * log(res.size)
+    return (res.index.size *
+            log((res.to_numpy() ** 2.0).sum() / res.index.size) +
+            nparam * log(res.index.size))
 
 
 def aic(obs=None, sim=None, res=None, missing="drop", nparam=1):
@@ -530,16 +476,17 @@ def aic(obs=None, sim=None, res=None, missing="drop", nparam=1):
         res = res.dropna()
 
     # Return nan if the time indices of the sim and obs don't match
-    if res.index.size is 0:
+    if res.index.size == 0:
         logger.warning("Time indices of the sim and obs don't match.")
         return nan
 
-    return -2.0 * log((res ** 2.0).sum()) + 2.0 * nparam
+    return (res.index.size *
+            log((res.to_numpy() ** 2.0).sum() / res.index.size) + 2.0 * nparam)
 
 
 # Forecast Error Metrics
 
-def kge_2012(obs, sim, missing="drop", weighted=False, max_gap=90):
+def kge_2012(obs, sim, missing="drop", weighted=False, max_gap=30):
     """Compute the (weighted) Kling-Gupta Efficiency (KGE).
 
     Parameters
@@ -553,11 +500,11 @@ def kge_2012(obs, sim, missing="drop", weighted=False, max_gap=90):
         supported now.
     weighted: bool, optional
         Weight the values by the normalized time step to account for
-        irregular time series. Default is True.
+        irregular time series. Default is False.
     max_gap: int, optional
         maximum allowed gap period in days to use for the computation of the
         weights. All time steps larger than max_gap are replace with the
-        mean weight. Default value is 90 days.
+        max_gap value. Default value is 30 days.
 
     Notes
     -----
@@ -583,7 +530,7 @@ def kge_2012(obs, sim, missing="drop", weighted=False, max_gap=90):
     sim = sim.reindex(obs.index).dropna()
 
     # Return nan if the time indices of the sim and obs don't match
-    if sim.index.size is 0:
+    if sim.index.size == 0:
         logger.warning("Time indices of the sim and obs don't match.")
         return nan
 
