@@ -1,13 +1,12 @@
-"""This module contains methods to compute the Standardized Groundwater Index.
-
-"""
+"""This module contains methods to compute the Standardized Groundwater
+Index."""
 
 from numpy import linspace
 from scipy.stats import norm
 
 
 def sgi(series):
-    """Method to compute the Standardized Groundwater Index.
+    """Method to compute the Standardized Groundwater Index [sgi_2013]_.
 
     Parameters
     ----------
@@ -21,10 +20,9 @@ def sgi(series):
 
     References
     ----------
-    .. [sgi_2013]: Bloomfield, J. P. and Marchant, B. P.: Analysis of
+    .. [sgi_2013] Bloomfield, J. P. and Marchant, B. P.: Analysis of
        groundwater drought building on the standardised precipitation index
        approach, Hydrol. Earth Syst. Sci., 17, 4769–4787, 2013.
-
     """
     series = series.copy()  # Create a copy to ensure series is untouched.
 
