@@ -12,27 +12,27 @@ Examples
 See Also
 --------
 pastas.model.Model.add_stressmodel
-
-
 """
 
 from logging import getLogger
 
 import numpy as np
-from pandas import date_range, Series, Timedelta, DataFrame, concat, Timestamp
+from pandas import DataFrame, Series, Timedelta, Timestamp, concat, date_range
 from scipy.signal import fftconvolve
+from scipy import __version__ as scipyversion
+from warnings import warn
 
-from .decorators import set_parameter, njit, PastasDeprecationWarning
+from .decorators import njit, set_parameter
 from .recharge import Linear
-from .rfunc import One, Exponential, HantushWellModel
+from .rfunc import Exponential, HantushWellModel, One
 from .timeseries import TimeSeries
-from .utils import validate_name
+from .utils import check_numba, validate_name
 
 logger = getLogger(__name__)
 
 __all__ = ["StressModel", "StressModel2", "Constant", "StepModel",
-           "LinearTrend", "FactorModel", "RechargeModel", "WellModel",
-           "TarsoModel"]
+           "LinearTrend", "RechargeModel", "WellModel", "TarsoModel",
+           "ChangeModel"]
 
 
 class StressModelBase:
@@ -44,7 +44,6 @@ class StressModelBase:
         Name of this stressmodel object. Used as prefix for the parameters.
     parameters: pandas.DataFrame
         Dataframe containing the parameters.
-
     """
     _name = "StressModelBase"
 
@@ -74,7 +73,6 @@ class StressModelBase:
         Notes
         -----
         The preferred method for parameter setting is through the model.
-
         """
         self.parameters.loc[name, 'initial'] = value
 
@@ -85,7 +83,6 @@ class StressModelBase:
         Notes
         -----
         The preferred method for parameter setting is through the model.
-
         """
         self.parameters.loc[name, 'pmin'] = value
 
@@ -96,7 +93,6 @@ class StressModelBase:
         Notes
         -----
         The preferred method for parameter setting is through the model.
-
         """
         self.parameters.loc[name, 'pmax'] = value
 
@@ -108,7 +104,6 @@ class StressModelBase:
         Notes
         -----
         The preferred method for parameter setting is through the model.
-
         """
         self.parameters.loc[name, 'vary'] = bool(value)
 
@@ -123,7 +118,6 @@ class StressModelBase:
         See Also
         --------
         ps.timeseries.TimeSeries.update_series
-
         """
         for stress in self.stress:
             stress.update_series(**kwargs)
@@ -144,7 +138,6 @@ class StressModelBase:
         -------
         data: dict
             dictionary with the dump of the stresses.
-
         """
         data = []
 
@@ -165,7 +158,6 @@ class StressModelBase:
         -------
         stress: pandas.Dataframe
             Pandas dataframe of the stress(es)
-
         """
         if tmin is None:
             tmin = self.tmin
@@ -184,7 +176,6 @@ class StressModelBase:
         data: dict
             dictionary with all necessary information to reconstruct the
             StressModel object.
-
         """
         data = {
             "stressmodel": self._name,
@@ -201,7 +192,7 @@ class StressModelBase:
             return len(self.stress)
 
     def _get_block(self, p, dt, tmin, tmax):
-        """Internal method to get the block-response function"""
+        """Internal method to get the block-response function."""
         if tmin is not None and tmax is not None:
             day = Timedelta(1, 'D')
             maxtmax = (Timestamp(tmax) - Timestamp(tmin)) / day
@@ -251,12 +242,12 @@ class StressModel(StressModelBase):
     --------
     pastas.rfunc
     pastas.timeseries.TimeSeries
-
     """
     _name = "StressModel"
 
     def __init__(self, stress, rfunc, name, up=True, cutoff=0.999,
                  settings=None, metadata=None, meanstress=None):
+
         if isinstance(stress, list):
             stress = stress[0]  # TODO Temporary fix Raoul, 2017-10-24
 
@@ -275,9 +266,7 @@ class StressModel(StressModelBase):
         self.set_init_parameters()
 
     def set_init_parameters(self):
-        """Set the initial parameters (back) to their default values.
-
-        """
+        """Set the initial parameters (back) to their default values."""
         self.parameters = self.rfunc.get_init_parameters(self.name)
 
     def simulate(self, p, tmin=None, tmax=None, freq=None, dt=1.0):
@@ -297,7 +286,6 @@ class StressModel(StressModelBase):
         -------
         pandas.Series
             The simulated head contribution.
-
         """
         self.update_stress(tmin=tmin, tmax=tmax, freq=freq)
         b = self._get_block(p, dt, tmin, tmax)
@@ -315,7 +303,6 @@ class StressModel(StressModelBase):
         data: dict
             dictionary with all necessary information to reconstruct the
             StressModel object.
-
         """
         data = {
             "stressmodel": self._name,
@@ -371,14 +358,20 @@ class StressModel2(StressModelBase):
     See Also
     --------
     pastas.rfunc
-    pastas.timeseries
-
+    pastas.timeseries.TimeSeries
     """
     _name = "StressModel2"
 
     def __init__(self, stress, rfunc, name, up=True, cutoff=0.999,
                  settings=("prec", "evap"), metadata=(None, None),
                  meanstress=None):
+
+        msg = "StressModel2 is deprecated. It will be removed in version " \
+              "0.22.0 and is replaced by the RechargeModel stress model. " \
+              "Please use ps.RechargeModel(prec, evap, " \
+              "recharge=ps.rch.Linear) for the same stress model."
+        warn(msg)
+
         # First check the series, then determine tmin and tmax
         stress0 = TimeSeries(stress[0], settings=settings[0],
                              metadata=metadata[0])
@@ -412,9 +405,7 @@ class StressModel2(StressModelBase):
         self.set_init_parameters()
 
     def set_init_parameters(self):
-        """Set the initial parameters back to their default values.
-
-        """
+        """Set the initial parameters back to their default values."""
         self.parameters = self.rfunc.get_init_parameters(self.name)
         self.parameters.loc[self.name + '_f'] = \
             (-1.0, -2.0, 0.0, True, self.name)
@@ -437,7 +428,6 @@ class StressModel2(StressModelBase):
         -------
         pandas.Series
             The simulated head contribution.
-
         """
         b = self._get_block(p[:-1], dt, tmin, tmax)
         stress = self.get_stress(p=p, tmin=tmin, tmax=tmax, freq=freq,
@@ -478,7 +468,6 @@ class StressModel2(StressModelBase):
         data: dict
             dictionary with all necessary information to reconstruct the
             StressModel object.
-
         """
         data = {
             "stressmodel": self._name,
@@ -517,7 +506,6 @@ class StepModel(StressModelBase):
     The step trend is calculated as follows. First, a binary series is
     created, with zero values before tstart, and ones after the start. This
     series is convoluted with the block response to simulate a step trend.
-
     """
     _name = "StepModel"
 
@@ -539,7 +527,7 @@ class StepModel(StressModelBase):
                                                       False, self.name)
 
     def simulate(self, p, tmin=None, tmax=None, freq=None, dt=1):
-        tstart = Timestamp.fromordinal(int(p[-1]), freq="D")
+        tstart = Timestamp.fromordinal(int(p[-1]))
         tindex = date_range(tmin, tmax, freq=freq)
         h = Series(0, tindex, name=self.name)
         h.loc[h.index > tstart] = 1
@@ -580,7 +568,6 @@ class LinearTrend(StressModelBase):
     While possible, it is not recommended to vary the parameters for the
     start and end time of the linear trend. These parameters are usually
     hard to impossible to estimate from the data.
-
     """
     _name = "LinearTrend"
 
@@ -645,7 +632,6 @@ class Constant(StressModelBase):
     initial: float, optional
         Initial estimate of the parameter value. E.g. The minimum of the
         observed series.
-
     """
     _name = "Constant"
 
@@ -701,7 +687,6 @@ class WellModel(StressModelBase):
     Warnings
     --------
     This model only works with the HantushWellModel response function.
-
     """
     _name = "WellModel"
 
@@ -709,11 +694,15 @@ class WellModel(StressModelBase):
                  settings="well", sort_wells=True):
         if not issubclass(rfunc, HantushWellModel):
             raise NotImplementedError("WellModel only supports the rfunc "
-                                      "HantushWellModel fow now!")
+                                      "HantushWellModel!")
 
-        logger.warning("It is recommended to use LmfitSolve as the solver "
-                       "when implementing WellModel. See "
-                       "https://github.com/pastas/pastas/issues/177.")
+        # Check if scipy < 1.8
+        from distutils.version import StrictVersion
+        if StrictVersion(scipyversion) < StrictVersion("1.8.0"):
+            logger.warning(
+                "It is recommended to use LmfitSolve as the solver "
+                "or update to scipy>=1.8.0 when implementing WellModel."
+                " See https://github.com/pastas/pastas/issues/177.")
 
         # sort wells by distance
         self.sort_wells = sort_wells
@@ -723,10 +712,11 @@ class WellModel(StressModelBase):
             if isinstance(settings, list):
                 settings = [s for _, s in sorted(zip(distances, settings),
                                                  key=lambda pair: pair[0])]
-            distances.sort()
+
+            distances = np.sort(distances)
 
         if settings is None or isinstance(settings, str):
-            settings = len(stress) * [None]
+            settings = len(stress) * [settings]
 
         # convert stresses to TimeSeries if necessary
         stress = self.handle_stress(stress, settings)
@@ -743,7 +733,8 @@ class WellModel(StressModelBase):
                                     name="distances")
 
         meanstress = np.max([s.series.std() for s in stress])
-        rfunc = rfunc(up=up, cutoff=cutoff, meanstress=meanstress)
+        rfunc = rfunc(up=up, cutoff=cutoff, meanstress=meanstress,
+                      distances=self.distances.values)
 
         tmin = np.min([s.series.index.min() for s in stress])
         tmax = np.max([s.series.index.max() for s in stress])
@@ -757,20 +748,12 @@ class WellModel(StressModelBase):
 
     def set_init_parameters(self):
         self.parameters = self.rfunc.get_init_parameters(self.name)
-        # ensure lambda can't get too small or too large
-        self.parameters.loc[self.name + "_b", "pmax"] /= \
-            np.max(self.distances) ** 2
-        self.parameters.loc[self.name + "_b", "pmin"] /= \
-            np.max(self.distances) ** 2
-        # set initial value with mean distance
-        self.parameters.loc[self.name + "_b", "initial"] /= \
-            np.mean(self.distances) ** 2
 
     def simulate(self, p=None, tmin=None, tmax=None, freq=None, dt=1,
                  istress=None, **kwargs):
         distances = self.get_distances(istress=istress)
         stress_df = self.get_stress(p=p, tmin=tmin, tmax=tmax, freq=freq,
-                                    istress=istress)
+                                    istress=istress, squeeze=False)
         h = Series(data=0, index=self.stress[0].series.index, name=self.name)
         for name, r in distances.iteritems():
             stress = stress_df.loc[:, name]
@@ -806,25 +789,24 @@ class WellModel(StressModelBase):
         -------
         stress: list
             return a list with the stresses transformed to pastas TimeSeries.
-
         """
         data = []
 
         if isinstance(stress, Series):
-            data.append(TimeSeries(stress, settings))
+            data.append(TimeSeries(stress, settings=settings))
         elif isinstance(stress, dict):
-            for i, value in enumerate(stress.values()):
-                data.append(TimeSeries(value, settings=settings[i]))
+            for i, (name, value) in enumerate(stress.items()):
+                data.append(TimeSeries(value, name=name, settings=settings[i]))
         elif isinstance(stress, list):
             for i, value in enumerate(stress):
                 data.append(TimeSeries(value, settings=settings[i]))
         else:
-            logger.error("Stress format is unknown. Provide a"
-                         "Series, dict or list.")
+            logger.error("Stress format is unknown. Provide a Series, "
+                         "dict or list.")
         return data
 
     def get_stress(self, p=None, tmin=None, tmax=None, freq=None,
-                   istress=None, **kwargs):
+                   istress=None, squeeze=True, **kwargs):
         if tmin is None:
             tmin = self.tmin
         if tmax is None:
@@ -833,13 +815,20 @@ class WellModel(StressModelBase):
         self.update_stress(tmin=tmin, tmax=tmax, freq=freq)
 
         if istress is None:
-            return DataFrame.from_dict({s.name: s.series for s in self.stress})
+            df = DataFrame.from_dict({s.name: s.series for s in self.stress})
+            if squeeze:
+                return df.squeeze()
+            else:
+                return df
         elif isinstance(istress, list):
             return DataFrame.from_dict(
                 {s.name: s.series for s in self.stress}
             ).iloc[:, istress]
         else:
-            return self.stress[istress].series.to_frame()
+            if squeeze:
+                return self.stress[istress].series
+            else:
+                return self.stress[istress].series.to_frame()
 
     def get_distances(self, istress=None):
         if istress is None:
@@ -851,14 +840,16 @@ class WellModel(StressModelBase):
 
     def get_parameters(self, model=None, istress=None):
         """ Get parameters including distance to observation point and
-        return as array (dimensions (nstresses, 4)).
+        return as array (dimensions = (nstresses, 4)).
 
         Parameters
         ----------
         model : pastas.Model, optional
-            if not None (default), use optimal model parameters
+            if provided, return optimal model parameters, else return
+            initial parameters
         istress : int, optional
-            if not None (default), return all parameters
+            if provided, return specific parameter set, else
+            return all parameters
 
         Returns
         -------
@@ -888,7 +879,6 @@ class WellModel(StressModelBase):
         data: dict
             dictionary with all necessary information to reconstruct the
             WellModel object.
-
         """
         data = {
             "stressmodel": self._name,
@@ -902,72 +892,53 @@ class WellModel(StressModelBase):
         }
         return data
 
+    def variance_gain(self, model, istress=None):
+        """Calculate variance of the gain for WellModel.
 
-@PastasDeprecationWarning
-class FactorModel(StressModelBase):
-    """Model that multiplies a stress by a single value.
+        Variance of the gain is calculated based on propagation of uncertainty
+        using optimal values and the variances of A and b and the covariance
+        between A and b.
 
-    Parameters
-    ----------
-    stress: pandas.Series or pastas.timeseries.TimeSeries
-        Stress which will be multiplied by a factor. The stress does not
-        have to be equidistant.
-    name: str, optional
-        String with the name of the stressmodel.
-    settings: dict or str, optional
-        Dict or String that is forwarded to the TimeSeries object created
-        from the stress.
-    metadata: dict, optional
-        Dictionary with metadata, forwarded to the TimeSeries object created
-        from the stress.
-
-    Warnings
-    --------
-    This stressmodel is deprecated and will be removed in a future version
-    of Pastas. Please use ps.StressModel with rfunc=ps.One instead. This
-    will yield the same result.
-
-    """
-    _name = "FactorModel"
-
-    def __init__(self, stress, name="factor", settings=None, metadata=None):
-        if isinstance(stress, list):
-            stress = stress[0]  # Temporary fix Raoul, 2017-10-24
-
-        stress = TimeSeries(stress, settings=settings, metadata=metadata)
-
-        tmin = stress.series_original.index.min()
-        tmax = stress.series_original.index.max()
-
-        StressModelBase.__init__(self, name=name, tmin=tmin, tmax=tmax)
-        self.value = 1.  # Initial value
-        self.stress = [stress]
-        self.set_init_parameters()
-
-    def set_init_parameters(self):
-        self.parameters.loc[self.name + "_f"] = (
-            self.value, -np.inf, np.inf, True, self.name)
-
-    def simulate(self, p=None, tmin=None, tmax=None, freq=None, dt=1):
-        self.update_stress(tmin=tmin, tmax=tmax, freq=freq)
-        return self.stress[0].series * p[0]
-
-    def to_dict(self, series=True):
-        """Method to export the StressModel object.
+        Parameters
+        ----------
+        model : pastas.Model
+            optimized model
+        istress : int or list of int, optional
+            index of stress(es) for which to calculate variance of gain
 
         Returns
         -------
-        data: dict
-            dictionary with all necessary information to reconstruct the
-            StressModel object.
+        var_gain : float
+            variance of the gain calculated from model results
+            for parameters A and b
+
+        See Also
+        --------
+        pastas.HantushWellModel.variance_gain
 
         """
-        data = {
-            "stressmodel": self._name,
-            "name": self.name,
-            "stress": self.dump_stress(series)
-        }
-        return data
+        if model.fit is None:
+            raise AttributeError("Model not optimized! Run solve() first!")
+        if self.rfunc._name != "HantushWellModel":
+            raise ValueError("Response function must be HantushWellModel!")
+        if model.fit.pcov.isna().all(axis=None):
+            model.logger.warn("Covariance matrix contains only NaNs!")
+
+        # get parameters and (co)variances
+        A = model.parameters.loc[self.name + "_A", "optimal"]
+        b = model.parameters.loc[self.name + "_b", "optimal"]
+        var_A = model.fit.pcov.loc[self.name + "_A", self.name + "_A"]
+        var_b = model.fit.pcov.loc[self.name + "_b", self.name + "_b"]
+        cov_Ab = model.fit.pcov.loc[self.name + "_A", self.name + "_b"]
+
+        if istress is None:
+            r = np.asarray(self.distances)
+        elif isinstance(istress, int) or isinstance(istress, list):
+            r = self.distances.iloc[istress]
+        else:
+            raise ValueError("Parameter 'istress' must be None, list or int!")
+
+        return self.rfunc.variance_gain(A, b, var_A, var_b, cov_Ab, r=r)
 
 
 class RechargeModel(StressModelBase):
@@ -980,7 +951,7 @@ class RechargeModel(StressModelBase):
         pandas.Series or pastas.timeseries object containing the
         precipitation series.
     evap: pandas.Series or pastas.timeseries.TimeSeries
-        pandas.Series or pastas.timeseries object containing the
+        pandas.Series or pastas.timeseries object containing the potential
         evaporation series.
     rfunc: pastas.rfunc class, optional
         Response function used in the convolution with the stress. Default
@@ -991,7 +962,7 @@ class RechargeModel(StressModelBase):
         String with the name of the recharge model. Options are: Linear (
         default), FlexModel and Berendrecht. These can be accessed through
         ps.rch.
-    temp: pandas.Series or pastas.TimeSeries, optional
+    temp: pandas.Series or pastas.timeseries.TimeSeries, optional
         pandas.Series or pastas.TimeSeries object containing the
         temperature series. It depends on the recharge model is this
         argument is required or not.
@@ -1012,8 +983,8 @@ class RechargeModel(StressModelBase):
     See Also
     --------
     pastas.rfunc
-    pastas.timeseries
-    pastas.rch
+    pastas.timeseries.TimeSeries
+    pastas.recharge
 
     Notes
     -----
@@ -1034,13 +1005,13 @@ class RechargeModel(StressModelBase):
     We recommend not to store a RechargeModel is a variable named `rm`. This
     name is already reserved in IPython to remove files and will cause
     problems later.
-
     """
     _name = "RechargeModel"
 
     def __init__(self, prec, evap, rfunc=Exponential, name="recharge",
                  recharge=Linear(), temp=None, cutoff=0.999,
-                 settings=("prec", "evap"), metadata=(None, None)):
+                 settings=("prec", "evap", "evap"),
+                 metadata=(None, None, None)):
         # Store the precipitation and evaporation time series
         self.prec = TimeSeries(prec, settings=settings[0],
                                metadata=metadata[0])
@@ -1062,13 +1033,13 @@ class RechargeModel(StressModelBase):
         # Store recharge object
         self.recharge = recharge
 
-        # Store a temperature time series if needed or set to None
-        if self.recharge.temp is True:
-            if temp is None:
-                msg = "Recharge module requires a temperature series. " \
-                      "No temperature series were provided"
-                raise TypeError(msg)
-            elif len(settings) < 3 or len(metadata) < 3:
+        # Store a temperature time series if provided/needed or set to None
+        if self.recharge.snow is True and temp is None:
+            msg = "Recharge model requires a temperature series. " \
+                  "No temperature series were provided"
+            raise TypeError(msg)
+        if temp is not None:
+            if len(settings) < 3 or len(metadata) < 3:
                 msg = "Number of values for the settings and/or metadata is " \
                       "incorrect."
                 raise TypeError(msg)
@@ -1125,7 +1096,6 @@ class RechargeModel(StressModelBase):
         See Also
         --------
         ps.timeseries.TimeSeries.update_series
-
         """
         self.prec.update_series(**kwargs)
         self.evap.update_series(**kwargs)
@@ -1155,11 +1125,10 @@ class RechargeModel(StressModelBase):
         Returns
         -------
         pandas.Series
-
         """
         if p is None:
             p = self.parameters.initial.values
-        b = self._get_block(p[:-self.recharge.nparam], dt, tmin, tmax)
+        b = self._get_block(p[:self.rfunc.nparam], dt, tmin, tmax)
         stress = self.get_stress(p=p, tmin=tmin, tmax=tmax, freq=freq,
                                  istress=istress).values
         name = self.name
@@ -1169,15 +1138,14 @@ class RechargeModel(StressModelBase):
                 # only happen when Linear is used as the recharge model
                 stress = stress * p[-1]
             if self.stress[istress].name is not None:
-                name = "{} ({})".format(self.name, self.stress[istress].name)
+                name = f"{self.name} ({self.stress[istress].name})"
 
         return Series(data=fftconvolve(stress, b, 'full')[:stress.size],
                       index=self.prec.series.index, name=name, fastpath=True)
 
     def get_stress(self, p=None, tmin=None, tmax=None, freq=None,
                    istress=None, **kwargs):
-        """Method to obtain the recharge stress calculated by the recharge
-        model.
+        """Method to obtain the recharge stress calculated by the model.
 
         Parameters
         ----------
@@ -1198,7 +1166,6 @@ class RechargeModel(StressModelBase):
             When no istress is selected, this return the estimated recharge
             flux that is convoluted with a response function on the
             "simulate" method.
-
         """
         if tmin is None:
             tmin = self.tmin
@@ -1216,10 +1183,8 @@ class RechargeModel(StressModelBase):
                 temp = None
             if p is None:
                 p = self.parameters.initial.values
-
             stress = self.recharge.simulate(prec=prec, evap=evap, temp=temp,
                                             p=p[-self.recharge.nparam:])
-
             return Series(data=stress, index=self.prec.series.index,
                           name="recharge", fastpath=True)
         elif istress == 0:
@@ -1230,7 +1195,7 @@ class RechargeModel(StressModelBase):
             return self.temp.series
 
     def get_water_balance(self, p=None, tmin=None, tmax=None, freq=None):
-        """Experimental method to obtain the water balance components.
+        """Method to obtain the water balance components.
 
         Parameters
         ----------
@@ -1266,7 +1231,6 @@ class RechargeModel(StressModelBase):
         >>> ml.solve()
         >>> wb = sm.get_water_balance(ml.get_parameters("rch"))
         >>> wb.plot(subplots=True)
-
         """
         if p is None:
             p = self.parameters.initial.values
@@ -1276,7 +1240,12 @@ class RechargeModel(StressModelBase):
         evap = self.get_stress(tmin=tmin, tmax=tmax, freq=freq,
                                istress=1).values
 
-        df = self.recharge.get_water_balance(prec=prec, evap=evap, temp=None,
+        if self.temp is not None:
+            temp = self.get_stress(tmin=tmin, tmax=tmax, freq=freq,
+                                   istress=2).values
+        else:
+            temp = None
+        df = self.recharge.get_water_balance(prec=prec, evap=evap, temp=temp,
                                              p=p[-self.recharge.nparam:])
         df.index = self.prec.series.index
         return df
@@ -1284,11 +1253,12 @@ class RechargeModel(StressModelBase):
     def to_dict(self, series=True):
         data = {
             "stressmodel": self._name,
-            "prec": self.prec.to_dict(),
-            "evap": self.evap.to_dict(),
+            "prec": self.prec.to_dict(series=series),
+            "evap": self.evap.to_dict(series=series),
             "rfunc": self.rfunc._name,
             "name": self.name,
             "recharge": self.recharge._name,
+            "recharge_kwargs": self.recharge.kwargs,
             "cutoff": self.rfunc.cutoff,
             "temp": self.temp.to_dict() if self.temp else None
         }
@@ -1339,12 +1309,12 @@ class TarsoModel(RechargeModel):
     .. [knotters_1999] Knotters, M. & De Gooijer, Jan G.. (1999). TARSO
        modeling of water table depths. Water Resources Research. 35.
        10.1029/1998WR900049.
-
     """
     _name = "TarsoModel"
 
     def __init__(self, prec, evap, oseries=None, dmin=None, dmax=None,
                  rfunc=Exponential, **kwargs):
+        check_numba()
         if oseries is not None:
             if dmin is not None or dmax is not None:
                 msg = 'Please specify either oseries or dmin and dmax'
@@ -1367,16 +1337,16 @@ class TarsoModel(RechargeModel):
         p0 = self.rfunc.get_init_parameters(self.name)
         one = One(meanstress=self.dmin + 0.5 * (self.dmax - self.dmin))
         pd0 = one.get_init_parameters(self.name).squeeze()
-        p0.loc['{}_d'.format(self.name)] = pd0
-        p0.index = ['{}0'.format(x) for x in p0.index]
+        p0.loc[f'{self.name}_d'] = pd0
+        p0.index = [f'{x}0' for x in p0.index]
 
         # parameters for the second drainage level
         p1 = self.rfunc.get_init_parameters(self.name)
         initial = self.dmin + 0.75 * (self.dmax - self.dmin)
         pd1 = Series({'initial': initial, 'pmin': self.dmin, 'pmax': self.dmax,
                       'vary': True, 'name': self.name})
-        p1.loc['{}_d'.format(self.name)] = pd1
-        p1.index = ['{}1'.format(x) for x in p1.index]
+        p1.loc[f'{self.name}_d'] = pd1
+        p1.index = [f'{x}1' for x in p1.index]
 
         # parameters for the recharge-method
         pr = self.recharge.get_init_parameters(self.name)
@@ -1400,14 +1370,14 @@ class TarsoModel(RechargeModel):
     def _check_stressmodel_compatibility(ml):
         """Internal method to check if no other stressmodels, a constants or a
         transform is used."""
-        msg = "A TarsoModel cannot be combined with {}. Either remove the" \
-              " TarsoModel or the {}."
+        msg = "A TarsoModel cannot be combined with %s. Either remove the" \
+              " TarsoModel or the %s."
         if len(ml.stressmodels) > 1:
-            logger.warning(msg.format("other stressmodels", "stressmodels"))
+            logger.warning(msg, "other stressmodels", "stressmodels")
         if ml.constant is not None:
-            logger.warning(msg.format("a constant", "constant"))
+            logger.warning(msg, "a constant", "constant")
         if ml.transform is not None:
-            logger.warning(msg.format("a transform", "transform"))
+            logger.warning(msg, "a transform", "transform")
 
     @staticmethod
     @njit
@@ -1448,7 +1418,7 @@ class TarsoModel(RechargeModel):
                     (d1 - d - r[i] * c) / (h0 - d - r[i] * c))
                 if dtdr > dt:
                     raise (Exception())
-                # change paraemeters
+                # change parameters
                 high = newhigh
                 if high:
                     S, a, c, d = S1, a_e, c_e, d_e
@@ -1458,3 +1428,231 @@ class TarsoModel(RechargeModel):
                 exp_a = np.exp(-(dt - dtdr) / a)
                 h[i] = (d1 - d) * exp_a + r[i] * c * (1 - exp_a) + d
         return h
+
+
+class ChangeModel(StressModelBase):
+    """Model where the response function changes from one to another over time.
+
+    Parameters
+    ----------
+    stress: pandas.Series
+        pandas Series object containing the stress.
+    rfunc1: rfunc class
+        response function used in the convolution with the stress.
+    rfunc2: rfunc class
+        response function used in the convolution with the stress.
+    name: str
+        name of the stress.
+    tchange: str
+        string with the approximate date of the change.
+    up: bool or None, optional
+        True if response function is positive (default), False if negative.
+        None if you don't want to define if response is positive or negative.
+    cutoff: float, optional
+        float between 0 and 1 to determine how long the response is (default
+        is 99% of the actual response time). Used to reduce computation times.
+    settings: dict or str, optional
+        the settings of the stress. This can be a string referring to a
+        predefined settings dict, or a dict with the settings to apply.
+        Refer to the docstring of pastas.Timeseries for further information.
+    metadata: dict, optional
+        dictionary containing metadata about the stress. This is passed onto
+        the TimeSeries object.
+
+    Notes
+    -----
+    This model is based on Obergfjell et al. (2019).
+
+    References
+    ----------
+    Obergfell, C., Bakker, M. and Maas, K. (2019), Identification and
+    Explanation of a Change in the Groundwater Regime using Time Series
+    Analysis. Groundwater, 57: 886-894. https://doi.org/10.1111/gwat.12891
+
+    """
+    _name = "ChangeModel"
+
+    def __init__(self, stress, rfunc1, rfunc2, name, tchange, up=True,
+                 cutoff=0.999, settings=None, metadata=None):
+        if isinstance(stress, list):
+            stress = stress[0]  # TODO Temporary fix Raoul, 2017-10-24
+
+        stress = TimeSeries(stress, settings=settings, metadata=metadata)
+
+        StressModelBase.__init__(self, name=name, rfunc=None,
+                                 tmin=stress.series.index.min(),
+                                 tmax=stress.series.index.max())
+        self.rfunc1 = rfunc1(up=up, cutoff=cutoff)
+        self.rfunc2 = rfunc2(up=up, cutoff=cutoff)
+        self.tchange = Timestamp(tchange)
+
+        self.freq = stress.settings["freq"]
+        self.stress = [stress]
+        self.set_init_parameters()
+
+    def set_init_parameters(self):
+        """Internal method to set the initial parameters."""
+        self.parameters = concat(
+            [self.rfunc1.get_init_parameters("{}_1".format(self.name)),
+             self.rfunc2.get_init_parameters("{}_2".format(self.name))])
+
+        tmin = Timestamp.min.toordinal()
+        tmax = Timestamp.max.toordinal()
+        tchange = self.tchange.toordinal()
+
+        self.parameters.loc[self.name + "_beta"] = (0., -np.inf, np.inf,
+                                                    True, self.name)
+        self.parameters.loc[self.name + "_tchange"] = (tchange, tmin, tmax,
+                                                       False, self.name)
+        self.parameters.name = self.name
+
+    def simulate(self, p, tmin=None, tmax=None, freq=None, dt=1.0):
+        self.update_stress(tmin=tmin, tmax=tmax, freq=freq)
+        rfunc1 = self.rfunc1.block(p[:self.rfunc1.nparam])
+        rfunc2 = self.rfunc2.block(
+            p[self.rfunc1.nparam:self.rfunc1.nparam + self.rfunc2.nparam])
+
+        stress = self.stress[0].series
+        npoints = stress.index.size
+        t = np.linspace(0, 1, npoints)
+        beta = p[-2] * npoints
+
+        sigma = stress.index.get_loc(
+            Timestamp.fromordinal(int(p[-1]))) / npoints
+        omega = 1 / (np.exp(beta * (t - sigma)) + 1)
+        h1 = Series(data=fftconvolve(stress, rfunc1, 'full')[:npoints],
+                    index=stress.index, name="1", fastpath=True)
+        h2 = Series(data=fftconvolve(stress, rfunc2, 'full')[:npoints],
+                    index=stress.index, name="1", fastpath=True)
+        h = omega * h1 + (1 - omega) * h2
+
+        return h
+
+
+class ReservoirModel(StressModelBase):
+    """Time series model consisting of a single reservoir with two stresses.
+    The first stress causes the head to go up and the second stress causes 
+    the head to go down.
+
+    Parameters
+    ----------
+    stress: list of pandas.Series or list of pastas.timeseries
+        list of two pandas.Series or pastas.timeseries objects containing the
+        stresses. Usually the first is the precipitation and the second the
+        evaporation.
+    name: str
+        Name of the stress
+    settings: list of dicts or strs, optional
+        The settings of the stresses. This can be a string referring to a
+        predefined settings dict, or a dict with the settings to apply.
+        Refer to the docstring of pastas.Timeseries for further information.
+        Default is ("prec", "evap").
+    metadata: list of dicts, optional
+        dictionary containing metadata about the stress. This is passed onto
+        the TimeSeries object.
+
+    Notes
+    -----
+    The order in which the stresses are provided is the order the metadata
+    and settings dictionaries or string are passed onto the TimeSeries
+    objects. By default, the precipitation stress is the first and the
+    evaporation stress the second stress.
+
+    See Also
+    --------
+    pastas.timeseries
+    """
+    _name = "ReservoirModel"
+
+    def __init__(self, stress, reservoir, name, meanhead,
+                 settings=("prec", "evap"), metadata=(None, None),
+                 meanstress=None):
+        # Set resevoir object
+        self.reservoir = reservoir(meanhead)
+
+        # Code below is copied from StressModel2 and may not be optimal
+        # Check the series, then determine tmin and tmax
+        stress0 = TimeSeries(stress[0], settings=settings[0],
+                             metadata=metadata[0])
+        stress1 = TimeSeries(stress[1], settings=settings[1],
+                             metadata=metadata[1])
+
+        # Select indices from validated stress where both series are available.
+        index = stress0.series.index.intersection(stress1.series.index)
+        if index.empty:
+            msg = ('The two stresses that were provided have no '
+                   'overlapping time indices. Please make sure the '
+                   'indices of the time series overlap.')
+            logger.error(msg)
+            raise Exception(msg)
+
+        # First check the series, then determine tmin and tmax
+        stress0.update_series(tmin=index.min(), tmax=index.max())
+        stress1.update_series(tmin=index.min(), tmax=index.max())
+
+        if meanstress is None:
+            meanstress = (stress0.series - stress1.series).std()
+
+        StressModelBase.__init__(self, name=name, tmin=index.min(),
+                                 tmax=index.max(), rfunc=None)
+        self.stress.append(stress0)
+        self.stress.append(stress1)
+
+        self.freq = stress0.settings["freq"]
+        self.set_init_parameters()
+
+    def set_init_parameters(self):
+        """Set the initial parameters back to their default values."""
+        self.parameters = self.reservoir.get_init_parameters(self.name)
+
+    def simulate(self, p, tmin=None, tmax=None, freq=None, dt=1, istress=None):
+        """Simulates the head contribution.
+
+        Parameters
+        ----------
+        p: array_like
+            array_like object with the values as floats representing the
+            model parameters.
+        tmin: str, optional
+        tmax: str, optional
+        freq: str, optional
+        dt: float, time step
+        istress: int, not used
+
+        Returns
+        -------
+        pandas.Series
+            The simulated head contribution.
+        """
+
+        stress = self.get_stress(tmin=tmin, tmax=tmax, freq=freq)
+        h = Series(data=self.reservoir.simulate(stress[0], stress[1], p),
+                   index=stress[0].index, name=self.name, fastpath=True)
+        return h
+
+    def get_stress(self, p=None, tmin=None, tmax=None, freq=None,
+                   istress=0, **kwargs):
+        if tmin is None:
+            tmin = self.tmin
+        if tmax is None:
+            tmax = self.tmax
+
+        self.update_stress(tmin=tmin, tmax=tmax, freq=freq)
+
+        return self.stress[0].series, self.stress[1].series
+
+    def to_dict(self, series=True):
+        """Method to export the StressModel object.
+
+        Returns
+        -------
+        data: dict
+            dictionary with all necessary information to reconstruct the
+            StressModel object.
+        """
+        pass
+
+    def _get_block(self, p, dt, tmin, tmax):
+        """Internal method to get the block-response function.
+        Cannot be used (yet?) since there is no block response"""
+        pass
