@@ -3,8 +3,9 @@ from logging import getLogger
 import pandas as pd
 from pandas.tseries.frequencies import to_offset
 
-from .utils import _get_stress_dt, _get_dt, _get_time_offset, \
-    timestep_weighted_resample
+from .rcparams import rcParams
+from .utils import (_get_dt, _get_stress_dt, _get_time_offset,
+                    timestep_weighted_resample)
 
 logger = getLogger(__name__)
 
@@ -44,36 +45,14 @@ class TimeSeries:
     To obtain the predefined TimeSeries settings, you can run the following
     line of code:
 
-    >>> ps.TimeSeries._predefined_settings
+    >>> ps.rcParams["timeseries"]
 
     See Also
     --------
     pastas.timeseries.TimeSeries.update_series
         For the individual options for the different settings.
-
     """
-    _predefined_settings = {
-        "oseries": {"fill_nan": "drop", "sample_down": "drop"},
-        "prec": {"sample_up": "bfill", "sample_down": "mean",
-                 "fill_nan": 0.0, "fill_before": "mean", "fill_after": "mean"},
-        "evap": {"sample_up": "bfill", "sample_down": "mean",
-                 "fill_before": "mean", "fill_after": "mean",
-                 "fill_nan": "interpolate"},
-        "well": {"sample_up": "bfill", "sample_down": "mean",
-                 "fill_nan": 0.0, "fill_before": 0.0, "fill_after": 0.0},
-        "waterlevel": {"sample_up": "interpolate", "sample_down": "mean",
-                       "fill_before": "mean", "fill_after": "mean",
-                       "fill_nan": "interpolate"},
-        "level": {"sample_up": "interpolate", "sample_down": "mean",
-                  "fill_before": "mean", "fill_after": "mean",
-                  "fill_nan": "interpolate"},
-        "flux": {"sample_up": "bfill", "sample_down": "mean",
-                 "fill_before": "mean", "fill_after": "mean",
-                 "fill_nan": 0.0},
-        "quantity": {"sample_up": "divide", "sample_down": "sum",
-                     "fill_before": "mean", "fill_after": "mean",
-                     "fill_nan": 0.0},
-    }
+    _predefined_settings = rcParams["timeseries"]
 
     def __init__(self, series, name=None, settings=None, metadata=None,
                  freq_original=None, **kwargs):
@@ -159,8 +138,7 @@ class TimeSeries:
             self.update_series(force_update=True, **self.settings)
 
     def __repr__(self):
-        """Prints a simple string representation of the time series.
-        """
+        """Prints a simple string representation of the time series."""
         return f"{self.__class__.__name__}" \
                f"(name={self.name}, " \
                f"freq={self.settings['freq']}, " \
@@ -174,10 +152,9 @@ class TimeSeries:
 
     @series_original.setter
     def series_original(self, series):
-        """Sets a new freq_original for the TimeSeries"""
+        """Sets a new freq_original for the TimeSeries."""
         if not isinstance(series, pd.Series):
-            raise TypeError("Expected a Pandas Series, got {}".format(
-                type(series)))
+            raise TypeError(f"Expected a Pandas Series, got {type(series)}")
         else:
             self._series_original = series
             # make sure that tmin and tmax and freq_original are set in validate_series
@@ -258,7 +235,6 @@ class TimeSeries:
         -----
         The method will validate if any of the settings is changed to
         determine if the series need to be updated.
-
         """
         if self._update_settings(**kwargs) or force_update:
             tmin = self.settings['tmin']
@@ -284,14 +260,13 @@ class TimeSeries:
         Parameters
         ----------
         other: float or pandas.Series
-
         """
         self._series = self.series.multiply(other)
         self._series_original = self.series_original.multiply(other)
         self.update_series(force_update=True)
 
     def _validate_series(self, series):
-        """ Validate user provided time series.
+        """Validate user provided time series.
 
         Parameters
         ----------
@@ -314,60 +289,56 @@ class TimeSeries:
         5. Find the frequency of the time series
         6. Handle duplicate indices, average if they exist
         7. drop nan-values (info message is provided by _fill_nan method)
-
         """
 
         # 1. Make sure the values are floats
         if not pd.api.types.is_float_dtype(series):
             series = series.astype(float)
-            logger.info(f"Time series {self.name} updated to dtype float.")
+            logger.info("Time series %s updated to dtype float.", self.name)
 
         # 2. Make sure the index is a datetimeindex
         if not pd.api.types.is_datetime64_dtype(series.index):
             series.index = pd.to_datetime(series.index)
-            logger.info(f"Time series index for {self.name} updated to "
-                        f"dtype datetime64.")
+            logger.info("Time series index for %s updated to dtype "
+                        "datetime64.", self.name)
 
         # 3. Make sure the index is increasing (also works for irregular dt)
         if not series.index.is_monotonic_increasing:
             series = series.sort_index()
-            logger.info(f"Time series index for {self.name} sorted to have "
-                        f"time increasing.")
+            logger.info("Time series index for %s sorted to have time "
+                        "increasing.", self.name)
 
         # 4. Drop nan-values at the beginning and end of the time series
         if series.first_valid_index() != series.index[0]:
             series = series.loc[series.first_valid_index():].copy(deep=True)
-            logger.info(f"Nan-values were removed at the start of the "
-                        f"time series {self.name}.")
+            logger.info("Nan-values were removed at the start of the time "
+                        "series %s.", self.name)
 
         if series.last_valid_index() != series.index[-1]:
             series = series.loc[:series.last_valid_index()].copy(deep=True)
-            logger.info(f"Nan-values were removed at the end of the "
-                        f"time series {self.name}.")
+            logger.info("Nan-values were removed at the end of the time "
+                        "series %s.", self.name)
 
-        # 5. Find the frequency of the time series
+        # 5. Find the frequency of the time series, always report a message
         if self.freq_original:
-            msg = f"User provided frequency for time series {self.name}: " \
-                  f"freq={self.freq_original}"
+            logger.info("User provided frequency for time series %s: freq="
+                        "%s", self.name, self.freq_original)
         elif pd.infer_freq(series.index):
             self.freq_original = pd.infer_freq(series.index)
-            msg = f"Inferred frequency for time series {self.name}: " \
-                  f"freq={self.freq_original}"
+            logger.info("Inferred frequency for time series %s: freq=%s",
+                        self.name, self.freq_original)
         elif self.settings["fill_nan"] != "drop":
-            msg = f"Cannot determine frequency of series {self.name}: " \
-                  f"freq=None. Resample settings are ignored and " \
-                  f"timestep_weighted_resample is used."
+            logger.info("Cannot determine frequency of series %s: freq=None. "
+                        "Resample settings are ignored and "
+                        "timestep_weighted_resample is used.", self.name)
         else:
-            msg = f"Cannot determine frequency of series {self.name}: " \
-                  f"freq=None. The time series is irregular."
-
-        logger.info(msg)  # Always report a message for the frequency
+            logger.info("Cannot determine frequency of series %s: freq=None. "
+                        "The time series is irregular.", self.name)
 
         # 6. Handle duplicate indices
         if not series.index.is_unique:
-            msg = f"duplicate time-indexes were found in the Time Series " \
-                  f"{self.name}. Values were averaged."
-            logger.warning(msg)
+            logger.warning("duplicate time-indexes were found in the time "
+                           "series %s. Values were averaged.", self.name)
             grouped = series.groupby(level=0)
             series = grouped.mean()
 
@@ -391,7 +362,6 @@ class TimeSeries:
         -------
         update: bool
             True if settings are changed and series need to be updated.
-
         """
         update = False
         for key, value in kwargs.items():
@@ -406,9 +376,7 @@ class TimeSeries:
         return update
 
     def _change_frequency(self, series):
-        """Method to change the frequency of the time series.
-
-        """
+        """Method to change the frequency of the time series."""
         freq = self.settings["freq"]
 
         # 1. If no freq string is present or is provided (e.g. Oseries)
@@ -433,15 +401,13 @@ class TimeSeries:
 
         # Drop nan-values at the beginning and end of the time series
         series = series.loc[
-                 series.first_valid_index():series.last_valid_index()]
+            series.first_valid_index():series.last_valid_index()]
 
         return series
 
     def _sample_up(self, series):
         """Resample the time series when the frequency increases (e.g. from
-        weekly to daily values).
-
-        """
+        weekly to daily values)."""
         method = self.settings["sample_up"]
         freq = self.settings["freq"]
 
@@ -464,12 +430,12 @@ class TimeSeries:
                 series = series.asfreq(freq)
                 series.fillna(method, inplace=True)
             else:
-                msg = f"Time Series {self.name}: User-defined option for " \
-                      f"sample_up {method} is not supported"
-                logger.warning(msg)
+                logger.warning("Time Series %s: User-defined option for  "
+                               "sample_up %s is not supported", self.name,
+                               method)
 
-        msg = f"Time Series {self.name} were sampled up using {method}."
-        logger.info(msg)
+        logger.info("Time Series %s were sampled up using %s.", self.name,
+                    method)
 
         return series
 
@@ -482,7 +448,6 @@ class TimeSeries:
         make sure the labels are still at the end of each period, and
         data at the right side of the bucket is included (see
         http://pandas.pydata.org/pandas-docs/stable/generated/pandas.Series.resample.html)
-
         """
         method = self.settings["sample_down"]
         freq = self.settings["freq"]
@@ -512,17 +477,16 @@ class TimeSeries:
         elif method == "max":
             series = series.resample(freq, **kwargs).max()
         else:
-            msg = f"Time Series {self.name}: User-defined option for " \
-                  f"sample_down {method} is not supported"
-            logger.warning(msg)
+            logger.warning("Time Series %s: User-defined option for sample "
+                           "down %s is not supported", self.name, method)
 
         # TODO: replace by adding offset to resample method with pandas 1.1.0
         if self.settings['time_offset'] > pd.Timedelta(0):
             # The offset is removed by the resample-method, so we add it again
             series = series.shift(1, freq=self.settings["time_offset"])
 
-        logger.info(f"Time Series {self.name} was sampled down to freq "
-                    f"{freq} with method {method}.")
+        logger.info("Time Series %s was sampled down to freq %s with method "
+                    "%s.", self.name, freq, method)
 
         return series
 
@@ -532,16 +496,13 @@ class TimeSeries:
         tindex = pd.date_range(series.index[0].ceil(freq) + time_offset,
                                series.index[-1], freq=freq)
         series = timestep_weighted_resample(series, tindex)
-        msg = f"Time Series {self.name} was sampled down to freq {freq} " \
-              f"with method timestep_weighted_resample."
-        logger.info(msg)
+        logger.info("Time Series %s was sampled down to freq %s with method "
+                    "timestep_weighted_resample.", self.name, freq)
         return series
 
     def _fill_nan(self, series):
         """Fill up the nan-values when present and a constant frequency is
-        required.
-
-        """
+        required."""
 
         method = self.settings["fill_nan"]
         freq = self.freq_original
@@ -549,7 +510,7 @@ class TimeSeries:
         if freq:
             series = series.asfreq(freq)
             n = series.isnull().values.sum()
-            if n is 0:
+            if n == 0:
                 pass
             elif method == "drop":
                 series.dropna(inplace=True)
@@ -560,23 +521,21 @@ class TimeSeries:
             elif isinstance(method, float):
                 series.fillna(method, inplace=True)
             else:
-                msg = f"Time Series {self.name}: User-defined option for " \
-                      f"fill_nan {method} is not supported."
-                logger.warning(msg)
+                logger.warning("Time Series %s: User-defined option for "
+                               "fill_nan %s is not supported.", self.name,
+                               method)
         else:
             method = "drop"
             n = series.isnull().values.sum()
-            series.dropna(inplace=True)
+            series = series.dropna()
         if n > 0:
-            logger.info(f"Time Series {self.name}: {n} nan-value(s) was/were "
-                        f"found and filled with: {method}.")
+            logger.info("Time Series %s: %s nan-value(s) was/were found and "
+                        "filled with: %s.", self.name, n, method)
 
         return series
 
     def _fill_before(self, series):
-        """Method to add a period in front of the available time series.
-
-        """
+        """Method to add a period in front of the available time series."""
         freq = self.settings["freq"]
         method = self.settings["fill_before"]
         tmin = self.settings["tmin"]
@@ -592,24 +551,22 @@ class TimeSeries:
 
             if method == "mean":
                 series.fillna(series.mean(), inplace=True)  # Default option
-                msg = f"Time Series {self.name} was extended to " \
-                      f"{series.index.min()} with the mean value of the " \
-                      f"time series."
+                logger.info("Time Series %s was extended to %s with the mean "
+                            "value of the time series.", self.name,
+                            series.index.min())
             elif isinstance(method, float):
                 series.fillna(method, inplace=True)
-                msg = f"Time Series {self.name} was extended to" \
-                      f" {series.index.min()} by adding {method} values."
+                logger.info("Time Series %s was extended to %s by adding %s "
+                            "values.", self.name, series.index.min(), method)
             else:
-                msg = f"Time Series {self.name}: User-defined option for " \
-                      f"fill_before {method} is not supported."
-            logger.info(msg)
+                logger.info("Time Series %s: User-defined option for "
+                            "fill_before %s is not supported.", self.name,
+                            method)
 
         return series
 
     def _fill_after(self, series):
-        """Method to add a period in front of the available time series.
-
-        """
+        """Method to add a period in front of the available time series."""
         freq = self.settings["freq"]
         method = self.settings["fill_after"]
         tmax = self.settings["tmax"]
@@ -625,29 +582,26 @@ class TimeSeries:
 
             if method == "mean":
                 series.fillna(series.mean(), inplace=True)  # Default option
-                msg = f"Time Series {self.name} was extended to " \
-                      f"{series.index.max()} with the mean value of the " \
-                      f"time series."
+                logger.info("Time Series %s was extended to %s with the mean "
+                            "value of the time series.", self.name,
+                            series.index.max())
             elif isinstance(method, float):
                 series.fillna(method, inplace=True)
-                msg = f"Time Series {self.name} was extended to " \
-                      f"{series.index.max()} by adding {method} values."
+                logger.info("Time Series %s was extended to %s by adding %s "
+                            "values.", self.name, series.index.max(), method)
             else:
-                msg = f"Time Series {self.name}: User-defined option for " \
-                      f"fill_after {method} is not supported"
-            logger.info(msg)
+                logger.info("Time Series %s: User-defined option for "
+                            "fill_after %s is not supported", self.name,
+                            method)
 
         return series
 
     def _normalize(self, series):
-        """Method to normalize the time series.
-
-        """
+        """Method to normalize the time series."""
         method = self.settings["norm"]
-        msg = f"Time series {self.name} is normalized with the {method}."
 
         if method is None:
-            msg = None
+            pass
         elif method == "mean":
             series = series.subtract(series.mean())
         elif method == "median":
@@ -659,10 +613,12 @@ class TimeSeries:
         elif isinstance(method, float):
             series = series.subtract(method)
         else:
-            msg = f"Time Series {self.name}: Selected method {method} to " \
-                  f"normalize the time series is not supported"
-        if msg:
-            logger.info(msg)
+            logger.warning("Time Series %s: Selected method %s to normalize "
+                           "the time series is not supported", self.name,
+                           method)
+        if method:
+            logger.info("Time series %s is normalized with the %s", self.name,
+                        method)
 
         return series
 
@@ -680,7 +636,6 @@ class TimeSeries:
         data: dict
             dictionary with the necessary information to recreate the
             TimeSeries object completely.
-
         """
         data = {}
 
@@ -708,7 +663,6 @@ class TimeSeries:
         Returns
         -------
         matplotlib.Axes
-
         """
 
         if original:
